using at.jku.ssw.Analyse;

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace at.jku.ssw.Analyse.GUI
{
	/// <summary>
	/// Dieses Panel stellt die Informationen zu einem NamespaceInfo Objekt dar.
	/// </summary>
	public class NamespaceInfoPanel : InfoPanel
	{
		private NamespaceInfo niDisp;
		private System.Windows.Forms.TabControl tabMain;
		private System.Windows.Forms.TabPage tpSummary;
		private System.Windows.Forms.ListView lvSummary;
		private System.Windows.Forms.ColumnHeader colSummaryName;
		private System.Windows.Forms.ColumnHeader colSummaryVal;
		private System.Windows.Forms.TabPage tpSumStats;
		private System.Windows.Forms.ColumnHeader colSumStatsName;
		private System.Windows.Forms.ColumnHeader colSumStatsVal;
		private System.Windows.Forms.TabPage tpStats;
		private System.Windows.Forms.TabPage tpMethod;
		private System.Windows.Forms.ListView lvStats;
		private System.Windows.Forms.ListView lvMeth;
		private System.Windows.Forms.ColumnHeader colMethName;
		private System.Windows.Forms.ColumnHeader colMethVal;
		private System.Windows.Forms.ListView lvSumStats;
		private System.Windows.Forms.ColumnHeader colStatsName;
		private System.Windows.Forms.ColumnHeader colStatsVal;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public NamespaceInfoPanel(NamespaceInfo ni, int tabIndex) : this(ni) {
			if (tabIndex != -1)
				tabMain.SelectedIndex = tabIndex;
		}

		public NamespaceInfoPanel(NamespaceInfo ni)
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			this.niDisp = ni;
			FillLists();
		}

		private void FillLists() {
			ListViewItem head;
			/******* Summary Tab *********/
			//Allgemeine Infos
			lvSummary.Items.AddRange(new ListViewItem[] {
				new ListViewItem(new String[] {"Name", niDisp.sName.ToString()}),
				new ListViewItem(new String[] {"Full Name", niDisp.getFullName().ToString()}),
				new ListViewItem(new String[] {"Class Count", niDisp.getCntClasses().ToString()}),
				new ListViewItem(new String[] {"Field Count", niDisp.getCntFields().ToString()})
				}
			);
			//Using Infos
			head = new ListViewItem(new String[] {"Usings:", niDisp.uiUsings.alUsings.Count.ToString()});
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lvSummary.Items.Add(head);
			foreach (String u in niDisp.uiUsings.alUsings)
				lvSummary.Items.Add(new ListViewItem(new String[] {"", u}));
			//Aliases
			head = new ListViewItem(new String[] {"Aliases:", niDisp.uiUsings.htAliases.Count.ToString()});
			head.Font = new Font(head.Font, head.Font.Style | FontStyle.Bold);
			lvSummary.Items.Add(head);
			foreach (DictionaryEntry de in niDisp.uiUsings.htAliases)
				lvSummary.Items.Add(new ListViewItem(new String[] {"", (String)de.Key + "->" + (String)de.Value}));
			//Variables
			FillVariableInfo(niDisp, lvSummary);

			colSummaryName.Width = -1; //autosize to the longest item
			colSummaryVal.Width = -1;
			/****** Statistic Tab ********/
			//LinesOfCode Infos
			FillLOCInfo(niDisp.locInfo, lvStats);
			//StatementCount Infos
			FillStatInfo(niDisp.statInfo, lvStats);

			colStatsName.Width = -1;
			colStatsVal.Width = -1;

			/****** Summed Statistic Tab ****/
			//LinesOfCode Infos
			FillLOCInfo(niDisp.locSumInfo, lvSumStats);
			//StatementCount Infos
			FillStatInfo(niDisp.statSumInfo, lvSumStats);

			colSumStatsName.Width = -1;
			colSumStatsVal.Width = -1;

			/****** Method Tab ******/
			FillMethodAvgInfo(niDisp, lvMeth);
			FillHalsteadInfos(niDisp.hInfo, lvMeth);

			colMethName.Width = -1;
			colMethVal.Width = -1;

		}

		public override int getTabIndex() {
			return tabMain.SelectedIndex;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.tabMain = new System.Windows.Forms.TabControl();
			this.tpSummary = new System.Windows.Forms.TabPage();
			this.lvSummary = new System.Windows.Forms.ListView();
			this.colSummaryName = new System.Windows.Forms.ColumnHeader();
			this.colSummaryVal = new System.Windows.Forms.ColumnHeader();
			this.tpStats = new System.Windows.Forms.TabPage();
			this.lvStats = new System.Windows.Forms.ListView();
			this.colStatsName = new System.Windows.Forms.ColumnHeader();
			this.colStatsVal = new System.Windows.Forms.ColumnHeader();
			this.tpSumStats = new System.Windows.Forms.TabPage();
			this.lvSumStats = new System.Windows.Forms.ListView();
			this.colSumStatsName = new System.Windows.Forms.ColumnHeader();
			this.colSumStatsVal = new System.Windows.Forms.ColumnHeader();
			this.tpMethod = new System.Windows.Forms.TabPage();
			this.lvMeth = new System.Windows.Forms.ListView();
			this.colMethName = new System.Windows.Forms.ColumnHeader();
			this.colMethVal = new System.Windows.Forms.ColumnHeader();
			this.tabMain.SuspendLayout();
			this.tpSummary.SuspendLayout();
			this.tpStats.SuspendLayout();
			this.tpSumStats.SuspendLayout();
			this.tpMethod.SuspendLayout();
			this.SuspendLayout();
			// 
			// tabMain
			// 
			this.tabMain.Controls.AddRange(new System.Windows.Forms.Control[] {
																				  this.tpSummary,
																				  this.tpStats,
																				  this.tpSumStats,
																				  this.tpMethod});
			this.tabMain.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabMain.Name = "tabMain";
			this.tabMain.SelectedIndex = 0;
			this.tabMain.Size = new System.Drawing.Size(408, 360);
			this.tabMain.TabIndex = 0;
			// 
			// tpSummary
			// 
			this.tpSummary.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.lvSummary});
			this.tpSummary.Location = new System.Drawing.Point(4, 22);
			this.tpSummary.Name = "tpSummary";
			this.tpSummary.Size = new System.Drawing.Size(400, 334);
			this.tpSummary.TabIndex = 0;
			this.tpSummary.Text = "Summary";
			// 
			// lvSummary
			// 
			this.lvSummary.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						this.colSummaryName,
																						this.colSummaryVal});
			this.lvSummary.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvSummary.FullRowSelect = true;
			this.lvSummary.GridLines = true;
			this.lvSummary.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvSummary.MultiSelect = false;
			this.lvSummary.Name = "lvSummary";
			this.lvSummary.Size = new System.Drawing.Size(400, 334);
			this.lvSummary.TabIndex = 1;
			this.lvSummary.View = System.Windows.Forms.View.Details;
			// 
			// colSummaryName
			// 
			this.colSummaryName.Text = "Name";
			this.colSummaryName.Width = 128;
			// 
			// colSummaryVal
			// 
			this.colSummaryVal.Text = "Value";
			this.colSummaryVal.Width = 140;
			// 
			// tpStats
			// 
			this.tpStats.Controls.AddRange(new System.Windows.Forms.Control[] {
																				  this.lvStats});
			this.tpStats.Location = new System.Drawing.Point(4, 22);
			this.tpStats.Name = "tpStats";
			this.tpStats.Size = new System.Drawing.Size(400, 334);
			this.tpStats.TabIndex = 2;
			this.tpStats.Text = "Statistics";
			// 
			// lvStats
			// 
			this.lvStats.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					  this.colStatsName,
																					  this.colStatsVal});
			this.lvStats.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvStats.FullRowSelect = true;
			this.lvStats.GridLines = true;
			this.lvStats.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvStats.MultiSelect = false;
			this.lvStats.Name = "lvStats";
			this.lvStats.Size = new System.Drawing.Size(400, 334);
			this.lvStats.TabIndex = 2;
			this.lvStats.View = System.Windows.Forms.View.Details;
			// 
			// colStatsName
			// 
			this.colStatsName.Text = "Name";
			this.colStatsName.Width = 128;
			// 
			// colStatsVal
			// 
			this.colStatsVal.Text = "Value";
			this.colStatsVal.Width = 140;
			// 
			// tpSumStats
			// 
			this.tpSumStats.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.lvSumStats});
			this.tpSumStats.Location = new System.Drawing.Point(4, 22);
			this.tpSumStats.Name = "tpSumStats";
			this.tpSumStats.Size = new System.Drawing.Size(400, 334);
			this.tpSumStats.TabIndex = 1;
			this.tpSumStats.Text = "Summed Statistics";
			// 
			// lvSumStats
			// 
			this.lvSumStats.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						 this.colSumStatsName,
																						 this.colSumStatsVal});
			this.lvSumStats.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvSumStats.FullRowSelect = true;
			this.lvSumStats.GridLines = true;
			this.lvSumStats.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvSumStats.MultiSelect = false;
			this.lvSumStats.Name = "lvSumStats";
			this.lvSumStats.Size = new System.Drawing.Size(400, 334);
			this.lvSumStats.TabIndex = 0;
			this.lvSumStats.View = System.Windows.Forms.View.Details;
			// 
			// colSumStatsName
			// 
			this.colSumStatsName.Text = "Name";
			this.colSumStatsName.Width = 128;
			// 
			// colSumStatsVal
			// 
			this.colSumStatsVal.Text = "Value";
			this.colSumStatsVal.Width = 140;
			// 
			// tpMethod
			// 
			this.tpMethod.Controls.AddRange(new System.Windows.Forms.Control[] {
																				   this.lvMeth});
			this.tpMethod.Location = new System.Drawing.Point(4, 22);
			this.tpMethod.Name = "tpMethod";
			this.tpMethod.Size = new System.Drawing.Size(400, 334);
			this.tpMethod.TabIndex = 3;
			this.tpMethod.Text = "Methods";
			// 
			// lvMeth
			// 
			this.lvMeth.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																					 this.colMethName,
																					 this.colMethVal});
			this.lvMeth.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvMeth.FullRowSelect = true;
			this.lvMeth.GridLines = true;
			this.lvMeth.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.lvMeth.MultiSelect = false;
			this.lvMeth.Name = "lvMeth";
			this.lvMeth.Size = new System.Drawing.Size(400, 334);
			this.lvMeth.TabIndex = 2;
			this.lvMeth.View = System.Windows.Forms.View.Details;
			// 
			// colMethName
			// 
			this.colMethName.Text = "Name";
			this.colMethName.Width = 128;
			// 
			// colMethVal
			// 
			this.colMethVal.Text = "Value";
			this.colMethVal.Width = 140;
			// 
			// RootInfoPanel
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.tabMain});
			this.Name = "RootInfoPanel";
			this.Size = new System.Drawing.Size(408, 360);
			this.tabMain.ResumeLayout(false);
			this.tpSummary.ResumeLayout(false);
			this.tpStats.ResumeLayout(false);
			this.tpSumStats.ResumeLayout(false);
			this.tpMethod.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
