using System;
using System.Collections;
using System.Text;

namespace at.jku.ssw.Analyse {
	/// <summary>
	/// Diese Klasse ist fr alle Informationen einer Methode zustndig.
	/// </summary>
	public class MethodInfo : CodeInfo {
		public ArrayList parameters;	//Liste der Parameter der Mehode
		public int iCntMethodCalled;	//Wie oft wurde die Methode von anderen Methode
										//in dieser Datei aufgerufen
		private int iPCurDepth = 0; //Variable zu Berechnung der Maximalen Verschachtelungtiefe, bei der
									//bei jedem Auftreten eines eingebetteten
									//Statements 1 dazuaddieren wird (ber iCurDepth).
		public int iCurDepth {
			set{
				iPCurDepth = value;
				if (iPCurDepth > iMaxDepth) iMaxDepth = iPCurDepth;
			}
			get{
				return iPCurDepth;
			}
		}
		public int iMaxDepth;	//Die maximale Verschachtelungtiefe
		public int iMcCabe;		//Komplexittsma nach McCabe
		//Variablen zur Berechnung von Rechenberg
		public float fSC, fEC, fDC;
		public float fRechenbergCC;
		public int iExpDepth = -1;	//diese Variable gibt den aktuellen Verschachtenlungsgrad in einer
									//Anweisung an, und wir mit -1 initialisiert, da dann einfach bei
									//jedem Vorkommen einer Expression 1 dazuaddiert werden kann.
		//Aufgerufenen Methoden
		public Hashtable htMethodCalls;
		//Benutzte Variablen in Ausdrcken
		public Hashtable htOperands;
		//Variablen zur Berechnung von Henry & Kafura
		private Hashtable htVarRead;
		private Hashtable htVarWritten;
		public float fHenryKafura;

		public MethodInfo(String sName, int iLine, CodeInfo parent) : base(sName, iLine, parent) {
			parameters = new ArrayList();
			htMethodCalls = new Hashtable();
			htOperands = new Hashtable();
			htVarRead = new Hashtable();
			htVarWritten = new Hashtable();
		}

		public void addParameter(VariableInfo parameter) {
			parameters.Add(parameter);
		}

		public int getCntParameters() {
			return parameters.Count;
		}

		public int getCntMethodCalls() {
			int ret = 0;
			foreach (DictionaryEntry de in htMethodCalls) {
				ret += (int) de.Value;
			}
			return ret;
		}

		/// <summary>
		/// Diese Methode wird vom Parser aufgerufen, wenn er Variablenzugriffe erkannt hat.
		/// Dabei wird zwischen Schreibenden und Lesendem Variablenzugriffen unterschieden 
		/// (mittels bRead). Diese Zugriffe werden fr die Berechnung von Henry & Kafura bentigt.
		/// </summary>
		public void addVarAcc(Hashtable htAdd, bool bRead) {
			Hashtable htTmp = bRead ? htVarRead : htVarWritten;
			foreach (DictionaryEntry de in htAdd)
				htTmp[de.Key] = 1;
		}

		public override String getFullName() {
			StringBuilder sb = new StringBuilder();
			sb.Append(base.getFullName());
			sb.AppendFormat("({0})", getCntParameters());
			return sb.ToString();
		}

		/// <summary>
		/// Bentigt zum Auffinden von Methoden im Informationsbaum.
		/// </summary>
		public override MethodInfo findMethod(String sName) {
			if (sName == getFullName()) return this;
			return base.findMethod(sName);
		}

		public override void compute() {
			if (!hasComputed) {
				base.compute();
				//*********************************************************
				//Berechnung des Komplexittsmaes nach McCabe
				int iCntIf = statInfo.htKeywords.Contains("if") ? (int) statInfo.htKeywords["if"] : 0;
				int iCntCase = statInfo.iCaseInSwitch;
				int iCntSelect = statInfo.htKeywords.Contains("select") ? (int) statInfo.htKeywords["select"] : 0;
				int iCntFor = statInfo.getCntFor() + statInfo.getCntForeach();
				int iCntWhile = statInfo.getCntWhile() + statInfo.getCntDo();

				//die Formel wie in der Doku beschrieben
				iMcCabe = (1+2*iCntIf+iCntCase+iCntSelect+2*(iCntFor+iCntWhile)) -
						(2+iCntIf+iCntSelect+iCntFor+iCntWhile) + 2;

				//*********************************************************
				//Berechnung des Koplexittsmaes nach Halstead

				//Die von Halstead definierten Kennzahlen werden in CodeInfo.compute berechnet

				//*********************************************************
				//Berechnung des Koplexittsmaes nach Henry & Kafura

				//lokale Variablen zhlen nicht zu den Data-Flow's
				foreach (VariableInfo vi in declaredVariables) {
					foreach (DictionaryEntry de in (Hashtable)htVarRead.Clone())
						if (((String)de.Key).StartsWith(vi.sName))
							htVarRead.Remove(de.Key);
					foreach (DictionaryEntry de in (Hashtable)htVarWritten.Clone())
						if (((String)de.Key).StartsWith(vi.sName))
							htVarWritten.Remove(de.Key);
				}

				fHenryKafura = (float)locInfo.iLoCode *
									(float)Math.Pow((double)htVarRead.Count * (double)htVarWritten.Count, 2.0);

				//*********************************************************
				//Berechnung des Koplexittsmaes nach Rechenberg
				
				//Statementkomplexitt(SC) und Ausdruckskomplexitt(EC) werden schon vom Parser ermittelt
				//Fr die Datenkomplexitt(DC) muss man fr alle verwendeten Variablen, die in
				//htOperants stehen, feststellen, wo sie definiert wurden, und sie demnach gewichten.
				//Die Summe der Gewichte ist die Datenkomplexitt.
				const float rechDC_Lokal = 1.0f;
				const float rechDC_Param = 2.0f;
				const float rechDC_Class = 3.0f;
				const float rechDC_Parent = 4.0f;
				foreach (DictionaryEntry de in htOperands) {
					bool bFound = false;
					//ist es eine lokale Variable?
					foreach (VariableInfo vi in declaredVariables) {
						if (((String)de.Key).StartsWith(vi.sName)) {
							fDC += rechDC_Lokal;
							bFound = true;
							break;
						}
					}
					//ist es ein Parameter?
					if (!bFound)
						foreach (VariableInfo vi in parameters) {
							if (((String)de.Key).StartsWith(vi.sName)) {
								fDC += rechDC_Param;
								bFound = true;
								break;
							}
						}
					//ist es eine Klassenvariable?
					if (!bFound) {
						String sTmp = (String)de.Key;
						if (sTmp.StartsWith("this.")) sTmp = sTmp.Substring(5);
						foreach (VariableInfo vi in parent.declaredVariables) {
							if (sTmp.StartsWith(vi.sName)) {
								fDC += rechDC_Class;
								bFound = true;
								break;
							}
						}
					}
					//ist es eine geerbte Variable?
					if (!bFound) {
						ClassInfo ciParClass = (ClassInfo) parent.findClassNFN(((ClassInfo)parent).parentClass);
						int iDepth = 0;
						while (ciParClass != null) {
							String sTmp = (String)de.Key;
							if (sTmp.StartsWith("base.")) sTmp = sTmp.Substring(5);
							foreach (VariableInfo vi in ciParClass.declaredVariables) {
								if (sTmp.StartsWith(vi.sName)) {
									fDC += rechDC_Parent + iDepth;
									bFound = true;
									break;
								}
							}
							iDepth++;
							ciParClass = ciParClass.findClassNFN(ciParClass.parentClass);
						}
					}
					
					//wir haben nichts gefunden, wir nehmen also den schlechtesten Fall, eine
					//geerbte Variable, an
					if (!bFound)
						fDC += rechDC_Parent;

				}

				fRechenbergCC = fSC + fEC + fDC;

				//*********************************************************
				//Suchen der Methoden, die diese Methode aufruft, und erhhen des "aufgerufen Zhlers"
				foreach (DictionaryEntry de in htMethodCalls) {
					String sMethName = "";
					String sClassName = "";
					int iInd = ((String) de.Key).LastIndexOf(".");
					ClassInfo ciSearchedClass = null;
					ClassInfo ciTmp = null;
					bool bVarFound = false;

					//Die Methode wurde im aktuellen Objekt aufgerufen
					if (iInd == -1) {
						sMethName = (String) de.Key;
						ciSearchedClass = (ClassInfo) parent;
					}
					else {
						sMethName = ((String) de.Key).Substring(iInd+1);
						sClassName = ((String) de.Key).Substring(0, iInd);
						if (sClassName.StartsWith("this")) { //Methodenaufruf im gleichen Objekt
							sClassName = sClassName.Substring(Math.Min(5, sClassName.Length));
							ciTmp = (ClassInfo) parent;
						}
						else if (sClassName.StartsWith("base")) { //Methodenaufruf im Vaterobjekt
							sClassName = sClassName.Substring(Math.Min(5, sClassName.Length));
							ciTmp = parent.findClassNFN(((ClassInfo)parent).parentClass);
						}
						else {
							//berprfen, ob es ein Methodenaufruf eines lokalen Objektes ist
							foreach(VariableInfo vi in declaredVariables) {
								if (sClassName.StartsWith(vi.sName)) {
									sClassName = sClassName.Substring(Math.Min(vi.sName.Length+1, sClassName.Length));
									ciTmp = vi.findTyp();
									bVarFound = true; break;
								}
							}
							//ein Methodenaufruf eines Parameters
							if (!bVarFound)
								foreach(VariableInfo vi in parameters) {
									if (sClassName.StartsWith(vi.sName)) {
										sClassName = sClassName.Substring(Math.Min(vi.sName.Length+1, sClassName.Length));
										ciTmp = vi.findTyp();
										bVarFound = true; break;
									}								
								}
							//oder eine Methodenaufruf einer Klassenvariable
							if (!bVarFound)
								foreach(VariableInfo vi in ((ClassInfo) parent).declaredVariables) {
									if (sClassName.StartsWith(vi.sName)) {
										sClassName = sClassName.Substring(Math.Min(vi.sName.Length+1, sClassName.Length));
										ciTmp = vi.findTyp();
										break;
									}
								}
						}
						//falls wir ein konstrukt wie var1.feld1.feld2.methode haben, mssen
						//wir noch weiter suchen
						bVarFound = true;
						while (ciTmp != null && sClassName != "") {
							bVarFound = false;
							foreach (VariableInfo vi in ciTmp.declaredVariables) {
								if (sClassName.StartsWith(vi.sName)) {
									sClassName = sClassName.Substring(Math.Min(vi.sName.Length+1, sClassName.Length));
									ciTmp = vi.findTyp();
									bVarFound = true; break;
								}
							}
							if (!bVarFound && sClassName != "") {
								ciTmp = null; break;
							}
						}
						//Vielleicht haben wir es mit einem Aufruf einer statischen Methode
						//in einer statischen Klasse zu tun. Also wird noch berprft, ob
						//es eine solche Klasse gibt.
						if (ciTmp == null && sClassName != "")
							ciTmp=parent.findClassNFN(sClassName);

						ciSearchedClass = ciTmp;
					}
					MethodInfo miTmp = null;
					while (ciSearchedClass != null && miTmp == null) {
						//wir haben die Klasse, in der die aufgerufenen 
						//Methode ist, gefunden
						miTmp = getRoot().findMethod(ciSearchedClass.getFullName() + "." + sMethName);
						//wenn wir jetzt die Methode auch finden, knnen wir ihren Zhler fr die
						//Anzahl der Aufrufe erhhen.
						if (miTmp != null) { miTmp.iCntMethodCalled += (int) de.Value; }
						else {
							//Falls wir keine Methode gefunden haben, knnte es sein, dass diese Methode
							//in einem Vaterobjekt definiert ist. Wir suchen also nach oben weiter
							ciSearchedClass = ciSearchedClass.findClassNFN(ciSearchedClass.parentClass);
						}
					}
				}
			}
		}
			
		public override String ToString(String sTab, bool bToDescend) {
			StringBuilder sb = new StringBuilder();
			sb.Append(sTab).Append("Method\n");
			sb.Append(sTab).Append("------\n");
			sb.Append(base.ToString(sTab, bToDescend));
			sb.Append(sTab).AppendFormat("McCabe:        {0}\n", iMcCabe);
			sb.Append(sTab).AppendFormat("Henry&Kafura:  {0}\n", fHenryKafura);
			sb.Append(sTab).AppendFormat("Rechenberg:    {0}\n", fRechenbergCC);
			sb.Append(sTab).AppendFormat("(SC={0}; EC={1}; DC={2})\n", fSC, fEC, fDC);
			sb.Append(sTab).AppendFormat("Max. Depth:    {0}\n", iMaxDepth);
			sb.Append(sTab).AppendFormat("Method Called: {0}\n", iCntMethodCalled);
			sb.Append(sTab).Append("Method calls: ").Append(getCntMethodCalls()).Append("\n");
			foreach (DictionaryEntry de in htMethodCalls) {
				sb.Append(sTab).Append(" ").Append(de.Value).Append(".\t:\t").Append(de.Key).Append("\n");
			}
			/*sb.Append(sTab).Append("Operands: ").Append("\n");
			foreach (DictionaryEntry de in htOperands) {
				sb.Append(sTab).Append(" ").Append(de.Value).Append(".\t:\t").Append(de.Key).Append("\n");
			}
			sb.Append(sTab).Append("Reads: ").Append("\n");
			foreach (DictionaryEntry de in htVarRead) {
				sb.Append(sTab).Append(" ").Append(de.Value).Append(".\t:\t").Append(de.Key).Append("\n");
			}
			sb.Append(sTab).Append("Writes: ").Append("\n");
			foreach (DictionaryEntry de in htVarWritten) {
				sb.Append(sTab).Append(" ").Append(de.Value).Append(".\t:\t").Append(de.Key).Append("\n");
			}*/
			sb.Append(sTab).Append("Parameter: ").Append(getCntParameters()).Append("\n");
			foreach (VariableInfo vi in parameters) {
				sb.Append(sTab).Append(" ").Append(vi.ToString()).Append("\n");
			}
			if (bToDescend)
				sb.Append(base.ToDescend(sTab));
			return sb.ToString();
		}

	}
}
